import os
from gettext import gettext as _
import deskbar
import deskbar.Handler
import deskbar.Utils
import leaftag
import gobject

HANDLERS = {
    "TagutilsHandler" : {
        "name":        _("Tags"),
        "description": _("Search for files and URIs by tag"),
    }
}

class SourceMatch(deskbar.Match.Match):
    def __init__(self, backend, source):
        uri = source.get_uri()
        schema = source.get_schema()
        path = source.get_filename()
        pixbuf = None
        if schema == "file":
            name = os.path.basename(path)
        else:
            name = uri

            if schema == "http":
                pixbuf = deskbar.Utils.load_icon("stock_internet")

        if pixbuf == None:
            pixbuf = deskbar.Utils.load_icon_for_file(uri)

        self._icon = pixbuf

        deskbar.Match.Match.__init__(self, backend, name=name)
        self._uri = uri
        self._source = source

    def action(self, text=None):
        gobject.spawn_async(["gnome-open", self._uri],
                            flags=gobject.SPAWN_SEARCH_PATH)

    def get_verb(self):
        if self._source.get_schema() == 'file':
            return _("Open %s") % "<b>%(name)s</b>"
        else:
            return _("Open the location %s") % "<b>%(name)s</b>"

    def get_hash(self, text=None):
        return self._uri

class TagMatch(deskbar.Match.Match):
    def __init__(self, backend, tag, **args):
        if type(tag) is leaftag.Tag:
            deskbar.Match.Match.__init__(self, backend, name=tag.get_name(), **args)
        else:
            deskbar.Match.Match.__init__(self, backend, name=tag, **args)

        self._tag = tag

    def action(self, text=None):
        gobject.spawn_async(["nautilus", "--no-desktop", "tags:///" + self._tag.get_name() + "/"], flags=gobject.SPAWN_SEARCH_PATH)

    def get_verb(self):
        return _("Search for all items tagged with %s") % \
               "<b>%(name)s</b>"

class TagutilsHandler(deskbar.Handler.Handler):
    def __init__(self):
        deskbar.Handler.Handler.__init__(self, None)

    def query(self, query, max=6):
        result = []
        sources = leaftag.lt_get_sources_with_tags([query], None)
        if len(sources) == 0:
            tags = leaftag.lt_get_all_tags()
            result += [TagMatch(self, tag) for tag in tags if tag.get_name().startswith(query)]
        else:
            for source in sources[:max]:
                if not source.get_schema() == 'file' or \
                   os.path.exists(source.get_filename()):
                    result.append(SourceMatch(self, source))

            if len(sources) > max:
                # If there are more results, show a "search for all files" match
                result.append(TagMatch(self, query))

        return result
